#include "Gadget/Engine.h" //This file includes all the other files
#include "Vehicle.h"
#include "Ship.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G; //Create an instance of the game engine
	G.Mouse.SetVisibility(false);
	G.Window.SetSize(1024, 768);
	//G.Window.SetFullScreen(true);
	G.Window.Open();
	G.Window.SetTitle("Cameras");
	glDisable(GL_DEPTH_TEST);

	Camera2D cam(&G);
	//cam.SetVibration(3.0f); 
	//cam.SetWobble(10.0f);
	cam.SetOffset(Vector2D(float(G.Window.GetWidth()/2), float(G.Window.GetHeight()/4)));

	Image levelImg[5];//The level is made from 5 textures
	levelImg[0].SetClamp(true);
	levelImg[0].Load("Images/GroundMat1.jpg"); //Load the image from disk.
	levelImg[1].SetClamp(true);
	levelImg[1].Load("Images/GroundMat2.jpg"); //Load the image from disk.
	levelImg[2].SetClamp(true);
	levelImg[2].Load("Images/GroundMat3.jpg"); //Load the image from disk.
	levelImg[3].SetClamp(true);
	levelImg[3].Load("Images/GroundMat4.jpg"); //Load the image from disk.
	levelImg[4].SetClamp(true);
	levelImg[4].Load("Images/GroundSolid.jpg"); //Load the image from disk.

	Sprite levelSpr;
	levelSpr.width   = 1024.0f; //Big tiles!
	levelSpr.height  = 1024.0f;
	
	//Here I add a vehicle using the vehicle class from the IA course
	Vehicle::G = &G;
	Ship ship;

	float zoom = 1.0f; //Controls the camera zoom

	while(G.Window.IsOpen()) //Loop continuously as long as the engine is running
	{
		G.Renderer.BeginFrame(); 
		G.Renderer.Render2D();

		if(G.Keyboard.GetKeyPress(GLFW_KEY_ESC)){G.Window.Close();}//If the escape key is pressed, exit the program
			
			Vector2D tempVec;
			if(G.Keyboard.GetKeyHold(GLFW_KEY_UP)) //Accelerate the car forward
			{
				tempVec = tempVec.AngleToVector(ship.direction);
			}

			//Rotate the car
			if(G.Keyboard.GetKeyHold(GLFW_KEY_RIGHT)) ship.direction -= 200.0f * G.Timer.GetElapsedTime();
			if(G.Keyboard.GetKeyHold(GLFW_KEY_LEFT)) ship.direction += 200.0f * G.Timer.GetElapsedTime();

			//Adjust the camera's scale
			if(G.Keyboard.GetKeyHold('=')) zoom += zoom * G.Timer.GetElapsedTime();
			if(G.Keyboard.GetKeyHold('-')) zoom -= zoom * G.Timer.GetElapsedTime();

			ship.SetSteeringForce(tempVec, tempVec.AngleToVector(ship.direction));//Look in the direction of thrust		
			ship.Step();   //Update position and direction

			//Update the camera
			cam.SetPosition(ship.position);
			cam.SetRotation(ship.direction-90.0f);
			cam.SetZoom(zoom);
			cam.Activate();

			//Render the ground tiles
			for(int y = 0; y<4; y++)
			{
				for(int x = 0; x<4; x++)
				{
					//Select the right image for this position
					if(x==1 && y==1)
					{
						levelImg[0].Activate();
					}
					else if(x==2 && y==1)
					{
						levelImg[1].Activate();
					}
					else if(x==1 && y==2)
					{
						levelImg[2].Activate();
					}
					else if(x==2 && y==2)
					{
						levelImg[3].Activate();
					}
					else
					{
						levelImg[4].Activate();
					}
					levelSpr.x = float(x) *1024.0f;
					levelSpr.y = float(-y)*1024.0f;
					levelSpr.Render();
				}
			}

			ship.Render();
			
			Shader::Deactivate(); //Always a good idea to deactivate shaders before refreshing the screen
		G.Renderer.EndFrame(); //Finished rendering, display the result
	}
	return 0;
}			